/**
 * <p>File: AlgebraicPolynomial.java</p>
 * <p>Title: Numerical methods</p>
 * <p>Description: Course on numerical analysis</p>
 * <p>Date: June 2012</p>
 * @author Faton M. Berisha
 * @version 7.1.0
 */

package numeric;

/**
 * Models an algebraic polynomial.
 * 
 */
public class AlgebraicPolynomial implements Differentiable {
	private double[] coefficient;
	private int degree;

	/**
	 * Creates an <code>AlgebraicPolynomial</code> with specified coefficients.
	 * 
	 * @param coefficient
	 *            coefficients
	 */
	public AlgebraicPolynomial(double[] coefficient) {
		this.coefficient = coefficient;
		degree = coefficient.length - 1;
	}

	/**
	 * Returns the specified coefficient of this
	 * <code>AlgebraicPolynomial</code>.
	 * 
	 * @param n
	 *            the power of the term whose coefficient is returned
	 * @return the specified coefficient
	 */
	public double getCoefficient(int n) {
		double coeff = 0;
		if (n >= 0 && n <= deg()) {
			coeff = coefficient[n];
		}
		return coeff;
	}

	/**
	 * Returns the coefficients of this <code>AlgebraicPolynomial</code>.
	 * 
	 * @return the <code>Matrix</code> coefficients
	 */
	public Matrix getCoefficient() {
		Matrix coeff = new Matrix(coefficient);
		return coeff;
	}

	/**
	 * Returns the degree of this <code>AlgebraicPolynomial</code>.
	 * 
	 * @return degree of this <code>AlgebraicPolynomial</code>
	 */
	public int getDegree() {
		return degree;
	}

	/**
	 * Sets the degree of this <code>AlgebraicPolynomial</code>.
	 * 
	 * @param degree
	 *            degree of this <code>AlgebraicPolynomial</code>
	 */
	public void setDegree(int degree) {
		this.degree = degree;
	}

	/**
	 * Returns the degree of this <code>AlgebraicPolynomial</code>.
	 * 
	 * @return degree of this <code>AlgebraicPolynomial</code>
	 */
	public int deg() {
		return getDegree();
	}

	/**
	 * Performs Horner's method on this <code>AlgebraicPolynomial</code>.
	 * Returns the <code>AlgebraicPolynomial</code> q such that p(x) == (x - x0)
	 * * q(x) + p(x0) for a specified value x0.
	 * 
	 * @param x0
	 *            the specified value
	 * @return <code>AlgebraicPolynomial</code> q
	 */
	public AlgebraicPolynomial horner(double x0) {
		int n = deg();
		double[] coeff = new double[n];
		coeff[n - 1] = getCoefficient(n);
		for (int i = n - 2; i >= 0; i--) {
			coeff[i] = x0 * coeff[i + 1] + getCoefficient(i + 1);
		}
		return new AlgebraicPolynomial(coeff);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see numeric.Function#value(double)
	 */
	public double value(double x) {
		return x * horner(x).getCoefficient(0) + getCoefficient(0);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see numeric.Differentiable#derivative(double)
	 */
	public double derivative(double x) {
		return horner(x).value(x);
	}

	/**
	 * Adds a specified <code>AlgebraicPolynomial</code> to this
	 * <code>AlgebraicPolynomial</code>.
	 * 
	 * @param p
	 *            the specified <code>AlgebraicPolynomial</code>
	 * @return the resulting <code>AlgebraicPolynomial</code>
	 */
	public AlgebraicPolynomial add(AlgebraicPolynomial p) {
		int n = Math.max(deg(), p.deg());
		double[] coeff = new double[n + 1];
		for (int i = 0; i <= n; i++) {
			coeff[i] = getCoefficient(i) + p.getCoefficient(i);
		}
		AlgebraicPolynomial r = new AlgebraicPolynomial(coeff);
		return r;
	}

	/**
	 * Multiplies a specified <code>AlgebraicPolynomial</code> to this
	 * <code>AlgebraicPolynomial</code>.
	 * 
	 * @param p
	 *            the specified <code>AlgebraicPolynomial</code>
	 * @return the resulting <code>AlgebraicPolynomial</code>
	 */
	public AlgebraicPolynomial multiply(AlgebraicPolynomial p) {
		int n = deg() + p.deg();
		double[] coeff = new double[n + 1];
		for (int i = 0; i <= n; i++) {
			for (int j = 0; j <= i; j++) {
				coeff[i] += getCoefficient(j) * p.getCoefficient(i - j);
			}
		}
		AlgebraicPolynomial r = new AlgebraicPolynomial(coeff);
		return r;
	}

	/**
	 * Returns the Lagrange's interpolating polynomial for a number of specified
	 * nodes.
	 * 
	 * @param m
	 *            a <code>Matrix</code> specifying the nodes as its columns
	 * @return the interpolating <code>AlgebraicPolynomial</code>
	 */
	public static AlgebraicPolynomial lagrange(Matrix m) {
		// TODO: Process exception if m.getNumcRows != 2
		// TODO: Process exception if m.getNumcCols == 0
		int n = m.getNumCols() - 1;
		AlgebraicPolynomial[] l = new AlgebraicPolynomial[n + 1];
		double[] x = new double[n + 1];
		for (int i = 0; i <= n; i++) {
			x[i] = m.get(0, i);
		}
		for (int i = 0; i <= n; i++) {
			l[i] = AlgebraicPolynomial.lagrangePolynomial(i, x);
		}
		AlgebraicPolynomial p = new AlgebraicPolynomial(new double[n + 1]);
		for (int i = 0; i <= n; i++) {
			double[] fxi = { m.get(1, i) };
			AlgebraicPolynomial f = new AlgebraicPolynomial(fxi);
			p = p.add(l[i].multiply(f));
		}
		return p;
	}

	/**
	 * Returns the Lagrange's interpolating polynomial for a number of specified
	 * nodes.
	 * 
	 * @param m
	 *            a <code>Matrix</code> specifying all the points as its columns
	 * @param index
	 *            an <code>int[]</code> specifying the positions of the points
	 *            to be used as interpolating nodes.
	 * @return the interpolating <code>AlgebraicPolynomial</code>
	 */
	public static AlgebraicPolynomial lagrange(Matrix m, int[] index) {
		// TODO: Process exception if m.getNumcRows != 2
		// TODO: Process exception if m.getNumcCols == 0
		int n = index.length - 1;
		double[] x = new double[n + 1];
		for (int i = 0; i <= n; i++) {
			x[i] = m.get(0, index[i]);
		}
		AlgebraicPolynomial[] l = new AlgebraicPolynomial[n + 1];
		for (int i = 0; i <= n; i++) {
			l[i] = AlgebraicPolynomial.lagrangePolynomial(i, x);
		}
		AlgebraicPolynomial p = new AlgebraicPolynomial(new double[n + 1]);
		for (int i = 0; i <= n; i++) {
			double[] fxi = { m.get(1, index[i]) };
			AlgebraicPolynomial f = new AlgebraicPolynomial(fxi);
			p = p.add(l[i].multiply(f));
		}
		return p;
	}

	/**
	 * Returns the Lagrange's interpolating polynomial for a number of specified
	 * nodes.
	 * 
	 * @param m
	 *            an <code>Matrix</code> specifying the nodes as its columns
	 * @return the interpolating <code>AlgebraicPolynomial</code>
	 */
	public static AlgebraicPolynomial interpolate(Matrix m) {
		return AlgebraicPolynomial.lagrange(m);
	}

	/**
	 * Evaluates the Lagrange's interpolating polynomial on a number of
	 * specified nodes at a specified point.
	 * 
	 * @param x
	 *            the specified point
	 * @param m
	 *            a <code>Matrix</code> specifying the nodes as its columns
	 * @return a <code>Matrix</code> obtained by the triangular scheme, with the
	 *         value at the last position of its diagonal
	 */
	public static Matrix neville(double x, Matrix m) {
		// TODO: Process exception if m.getNumcRows != 2
		// TODO: Process exception if m.getNumcCols == 0
		int n = m.getNumCols() - 1;
		Matrix q = new Matrix(n + 1, n + 1);
		for (int i = 0; i <= n; i++) {
			q.set(i, 0, m.get(1, i));
		}
		for (int i = 1; i <= n; i++) {
			for (int j = 1; j <= i; j++) {
				q.set(i, j, ((x - m.get(0, i - j)) * q.get(i, j - 1)
						- (x - m.get(0, i)) * q.get(i - 1, j - 1))
						/ (m.get(0, i) - m.get(0, i - j)));
			}
		}
		return q;
	}

	private static AlgebraicPolynomial lagrangePolynomial(int i, double[] x) {
		int n = x.length - 1;
		double[] one = { 1 };
		AlgebraicPolynomial prod = new AlgebraicPolynomial(one);
		for (int j = 0; j <= n; j++) {
			if (j != i) {
				double d = 1.0 / (x[i] - x[j]);
				double[] coeff = { -x[j] * d, d };
				prod = prod.multiply(new AlgebraicPolynomial(coeff));
			}
		}
		return prod;
	}
}
