/**
 * <p>File: Matrix.java</p>
 * <p>Title: Numerical methods</p>
 * <p>Description: Course on numerical analysis</p>
 * <p>Date: December 2011</p>
 * @author Faton M. Berisha
 * @version 6.0.0
 */

package numeric;

import java.util.Observable;

/**
 * Models a matrix.
 */
public class Matrix extends Observable implements Cloneable {

//	private static final long serialVersionUID = 2;

	private double[][] matrix;
	private int numRows;
	private int numCols;
	private boolean demo = false;

	/**
	 * Creates a <code>Matrix</code> with specified number of rows and
	 * columns.
	 * 
	 * @param numRows
	 *            number of rows
	 * @param numCols
	 *            number of columns
	 */
	public Matrix(int numRows, int numCols) {
		setSize(numRows, numCols);
	}

	/**
	 * Creates a <code>Matrix</code> by specified <code>double[][]</code>
	 * matrix.
	 * 
	 * @param val
	 *            the <code>double[][]</code> matrix with elements values.
	 */
	public Matrix(double[][] val) {
		setSize(val.length, val[0].length);
		for (int i = 0; i < numRows; i++) {
			for (int j = 0; j < numCols; j++) {
				matrix[i][j] = val[i][j];
			}
		}
	}

	/**
	 * Creates a column vector <code>Matrix</code>.
	 * 
	 * @param val
	 *            the <code>double[]</code> vector with elements values.
	 */
	public Matrix(double[] val) {
		setSize(val.length, 1);
		for (int i = 0; i < numRows; i++) {
			matrix[i][0] = val[i];
		}
	}

	/**
	 * Sets new dimensions of Matrix.
	 * 
	 * @param numRows
	 *            number of rows
	 * @param numCols
	 *            number of columns
	 */
	public void setSize(int numRows, int numCols) {
		if (numRows > 0 && numCols > 0) {
			this.numRows = numRows;
			this.numCols = numCols;
			matrix = new double[numRows][numCols];
		} else {
			throw new IllegalArgumentException("Illegal matrix dimensions: "
					+ numRows + "," + numCols);
		}
	}

	/**
	 * Gets the number of rows.
	 * 
	 * @return number of rows
	 */
	public int getNumRows() {
		return numRows;
	}

	/**
	 * Gets the number of columns.
	 * 
	 * @return number of columns
	 */
	public int getNumCols() {
		return numCols;
	}

	/**
	 * Gets if the mode is demo.
	 * 
	 * @return true if the mode is demo, false otherwise
	 */
	public boolean isDemo() {
		return demo;
	}

	/**
	 * Sets if the mode is demo.
	 * 
	 * @param demo
	 *            specifies if the mode is demo
	 */
	public void setDemo(boolean demo) {
		this.demo = demo;
	}

	/**
	 * Returns the element at the specified position in this <code>Matrix</code>.
	 * 
	 * @param row
	 *            row of the element to return
	 * @param col
	 *            column of the element to return
	 * @return the element at the specified position in this <code>Matrix</code>
	 */
	public double get(int row, int col) {
		return matrix[row][col];
	}

	/**
	 * Replaces the element at the specified position in this
	 * <code>Matrix</code> with the specified element.
	 * 
	 * @param row
	 *            row of the element to replace
	 * @param col
	 *            column of the element to replace
	 * @param val
	 *            value to be storede at the specified position
	 */
	public void set(int row, int col, double val) {
		matrix[row][col] = val;
	}

	/**
	 * Interchanges specified rows in this <code>Matrix</code>.
	 * 
	 * @param row1
	 *            specified row to be interchanged
	 * @param row2
	 *            the other specified row to be interchanged
	 */
	public void interchangeRows(int row1, int row2) {
		double[] temp = matrix[row1];
		matrix[row1] = matrix[row2];
		matrix[row2] = temp;
	}

	/**
	 * Permutates rows in this <code>Matrix</code> by the specified rows
	 * permutation.
	 * 
	 * @param permutation
	 *            specified rows permutaton
	 */
	public void permutateRows(int[] permutation) {
		double[][] temp = new double[numRows][];
		for (int i = 0; i < numRows; i++) {
			temp[i] = matrix[permutation[i]];
		}
		matrix = temp;
	}

	/**
	 * Returns a <code>Matrix</code> whose value is
	 * <code>(this - subtrahend)</code>.
	 * 
	 * @param subtrahend
	 *            value to be subtracted from this <code>Matrix</code>
	 * @return <code>(this - subtrahend)</code>
	 */
	public Matrix subtract(Matrix subtrahend) {
		if (numRows != subtrahend.numRows || numCols != subtrahend.numCols) {
			throw new IllegalArgumentException("Illegal matrix dimensions");
		}
		Matrix answer = new Matrix(numRows, numCols);
		for (int i = 0; i < numRows; i++) {
			for (int j = 0; j < numCols; j++) {
				answer.matrix[i][j] = matrix[i][j] - subtrahend.matrix[i][j];
			}
		}
		return answer;
	}

	/**
	 * Returns a norm of this <code>Matrix</code>.
	 * 
	 * @return a norm of this <code>Matrix</code>
	 */
	public double norm() {
		double max = 0.0;
		for (int i = 0; i < numRows; i++) {
			double sum = 0.0;
			for (int j = 0; j < numCols; j++) {
				sum += Math.abs(matrix[i][j]);
			}
			if (sum > max) {
				max = sum;
			}
		}
		return max;
	}

	/**
	 * Performs Gauss elimination to the system matrix.
	 */
	public void gauss() {
		demonstrate(this);
		int diagLength = Math.min(numRows, numCols);
		for (int diag = 0; diag < diagLength - 1; diag++) {
			int pivot = pivoting(diag);
			if (pivot != diag) {
				interchangeRows(pivot, diag);
			}
			for (int row = diag + 1; row < numRows; row++) {
				double mul = get(row, diag) / get(diag, diag);
				for (int col = diag + 1; col < numCols; col++) {
					double val = get(row, col) - get(diag, col) * mul;
					set(row, col, val);
				}
				set(row, diag, 0);
			}
			demonstrate(this);
		}
		if (get(diagLength - 1, diagLength - 1) == 0) {
			if (isDemo()) {
				deleteObservers();
			}
			throw new ArithmeticException("Singular matrix");
		}
	}

	/**
	 * Performs Gauss-Jordan elimination to the system matrix.
	 */
	public void gaussJordan() {
		int numRows = getNumEquations();
		int numCols = getNumCols();
		int diagLength = Math.min(numRows, numCols - 1);
		for (int diag = 0; diag < diagLength; diag++) {
			int pivot = pivoting(diag);
			if (pivot != diag)
				interchangeRows(pivot, diag);
			for (int row = 0; row < numRows; row++) {
				if (row != diag) {
					double mul = get(row, diag) / get(diag, diag);
					for (int col = diag + 1; col < numCols; col++) {
						double val = get(row, col) - get(diag, col) * mul;
						set(row, col, val);
					}
					set(row, diag, 0);
				}
			}
		}
		demonstrate(this);
	}

	public int getNumEquations() {
		return getNumRows();
	}

	public int getNumVariables() {
		return getNumCols();
	}

	public String toString() {
		StringBuffer sb = new StringBuffer();
		int rows = numRows;
		int cols = numCols;
		for (int i = 0; i < rows; i++) {
			sb.append(i == 0 ? "{{" : "{");
			for (int j = 0; j < cols - 1; j++) {
				sb.append(matrix[i][j] + ", ");
			}
			sb.append(matrix[i][cols - 1]);
			sb.append(i != rows - 1 ? "}" : "}}");
		}
		return sb.toString();
	}

	public Object clone() throws CloneNotSupportedException {
		Matrix answer = new Matrix(numRows, numCols);
		for (int i = 0; i < numRows; i++) {
			for (int j = 0; j < numCols; j++) {
				answer.set(i, j, get(i, j));
			}
		}
		return answer;
	}

	/**
	 * Performs pivoting for choosing the row of a next diagonal element in an
	 * elimination method.
	 * 
	 * @param diag
	 *            the specified diagonal element
	 * @return the chosen row of the next diagonal element
	 */
	protected int pivoting(int diag) {
		return noPivoting(diag);
		//return partialPivoting(diag);
	}

	/**
	 * Chooses the row of the first non-zero diagonal element in an elimination
	 * method.
	 * 
	 * @param diag
	 *            the specified diagonal element
	 * @return the chosen row of the next diagonal element
	 */
	protected int noPivoting(int diag) {
		int pivot = diag;
		while (get(pivot, diag) == 0) {
			pivot++;
		}
		if (pivot >= getNumRows()) {
			if (isDemo()) {
				deleteObservers();
			}
			throw new ArithmeticException("Singular matrix");
		}
		return pivot;
	}

	/**
	 * Performs partial pivoting for choosing the row of a next diagonal element
	 * in an elimination method.
	 * 
	 * @param diag
	 *            the specified diagonal element
	 * @return the chosen row of the next diagonal element
	 */
	protected int partialPivoting(int diag) {
		int pivot = diag;
		double max = Math.abs(get(diag, diag));
		int numRows = getNumEquations();
		for (int row = diag + 1; row < numRows; row++) {
			double temp = Math.abs(get(row, diag));
			if (temp > max) {
				pivot = row;
				max = temp;
			}
		}
		if (max == 0) {
			if (isDemo()) {
				deleteObservers();
			}
			throw new ArithmeticException("Singular matrix");
		}
		return pivot;
	}

	/**
	 * Notifies the observers to demonstrate this model's changes.
	 * 
	 * @param obj
	 *            specifies the object to be demonstrated
	 */
	protected void demonstrate(Object obj) {
		if (isDemo()) {
			setChanged();
			notifyObservers(obj);
		}
	}
}
